package com.napa.pulse.dao.impl;

import com.napa.pulse.dto.HubSpokeNetworkDTO;
import com.napa.pulse.dto.HubSpokeWidgetData;
import com.napa.pulse.entity.security.User;
import org.junit.Test;
import org.junit.jupiter.api.BeforeEach;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.mockito.junit.MockitoJUnitRunner;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.jdbc.core.JdbcTemplate;

import java.util.Arrays;
import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;

/**
 * @author Rahul Chaudhary
 * 7/12/2022
 */
@SpringBootTest
@RunWith(MockitoJUnitRunner.class)
public class HubSpokeDAOImplUnitTest {
    @InjectMocks
    HubSpokeNetworkDAOImpl hubSpokeNetworkDAO;

    @Mock
    private JdbcTemplate jdbcTemplate;

    @BeforeEach
    public void setup() {
        MockitoAnnotations.initMocks(this);
    }

    @Test
    public void test_getNetworkQueryByNetworkId() {
        Integer networkId = 101;
        String result = hubSpokeNetworkDAO.getNetworkQueryByNetworkId(networkId);
        String expected = "select hs.hs_network_id \"networkId\", hs.hs_network_name \"networkName\", PU.first_name || ' ' || PU.last_name \"netCreatedBy\", CASE WHEN HS.MODIFIED_DATE IS NULL THEN HS.CREATED_DATE ELSE HS.MODIFIED_DATE END \"createdDate\", " +
                "hs.unique_item_count_orig \"originalUIC\" from HUB_SPOKE_NETWORK HS INNER JOIN PULSE_USER PU ON PU.USER_ID=CASE WHEN HS.MODIFIED_BY IS NULL THEN HS.CREATED_BY ELSE HS.MODIFIED_BY END WHERE HS.HS_NETWORK_ID =" + networkId + " AND HS.IS_ACTIVE=1";
        assertThat(result).isNotNull();
//        assertThat(result).isEqualTo(expected);
    }

    @Test
    public void test_getNetworkById_when_incorrect_networkID() {
        List<HubSpokeNetworkDTO> list = hubSpokeNetworkDAO.getNetworkById(101, null);
        assertThat(list.size()).isEqualTo(0);
        list = hubSpokeNetworkDAO.getNetworkById(null, null);
        assertThat(list.size()).isEqualTo(0);
    }

    @Test
    public void test_getNetworkById_when_correct_networkID() {
        List<HubSpokeNetworkDTO> list = hubSpokeNetworkDAO.getNetworkById(680, buildUser());
        assertThat(list.size()).isEqualTo(0);
    }

    @Test
    public void test_getThresholdValue_when_allArguementsArePassed() {
        Integer result = hubSpokeNetworkDAO.getThresholdValue(101, 101);
        assertThat(result).isEqualTo(4);
    }

    @Test
    public void test_getThresholdValue_when_noArguementsArePassed() {
        Integer result = hubSpokeNetworkDAO.getThresholdValue(null, null);
        assertThat(result).isEqualTo(4);
    }

    private HubSpokeWidgetData buildHubSpokeWidgetData() {
        HubSpokeWidgetData obj = new HubSpokeWidgetData();
        obj.setUser(buildUser());
        obj.setSelectedHSFilterLoad("showNonEmptyHSNetwork");
        return obj;
    }

    private HubSpokeNetworkDTO buildHubSpokeNetworkDTO() {
        HubSpokeNetworkDTO obj = new HubSpokeNetworkDTO();
        obj.setNetworkId(101);
        obj.setNetworkName("Test-000");
        obj.setNetworkCreatedBy("Rahul Chaudhary");
        obj.setNetworkCreatedDate(null);
        obj.setCurrentUIC(0);
        obj.setCurrentUIC(0);
        obj.setChangeUIC(0);
        obj.setExpanded(false);
        return obj;
    }

    private User buildUser() {
        User testUser = new User();
        testUser.setUserId(8194);
        testUser.setFirstName("Test");
        testUser.setLastName("user");
        testUser.setLogin("login");
        testUser.setRoleId(1);
        testUser.setAccessItemIds(Arrays.asList(1, 2, 3));
        testUser.setLanguage("EN");
        testUser.setBusinessUnit(1);
        testUser.setDeactivated("N");
        testUser.setSiteTypes(Arrays.asList("dc", "sku", "ab"));

        return testUser;
    }
}
