
package com.napa.pulse.service.impl;

import com.napa.pulse.dto.FiledLineGroupCode;

import com.napa.pulse.dto.LeadTimeData;

import com.napa.pulse.entity.pulseui.MinMaxClacLn;

import com.napa.pulse.repository.LeadTimeRepository;

import com.napa.pulse.repository.MinMaxSitePrefRepository;

import com.napa.pulse.repository.SiteRepository;

import com.napa.pulse.security.GCPAuthenticationService;

import org.junit.jupiter.api.BeforeEach;

import org.junit.jupiter.api.Test;

import org.junit.jupiter.api.extension.ExtendWith;

import org.mockito.InjectMocks;

import org.mockito.Mock;

import org.mockito.MockitoAnnotations;

import org.mockito.junit.jupiter.MockitoExtension;

import org.slf4j.Logger;

import org.slf4j.LoggerFactory;

import org.springframework.http.*;

import org.springframework.jdbc.core.JdbcTemplate;

import org.springframework.test.util.ReflectionTestUtils;

import org.springframework.web.client.RestTemplate;

import java.util.*;

import static org.junit.Assert.assertEquals;

import static org.junit.jupiter.api.Assertions.*;

import static org.mockito.ArgumentMatchers.*;

import static org.mockito.Mockito.*;

@ExtendWith(MockitoExtension.class)

public class LeadTimeServiceImplUnitTest {

    @Mock

    private GCPAuthenticationService gcpAuthenticationService;

    @Mock

    private RestTemplate restTemplate;

    @Mock

    private SiteRepository siteRepo;

    @Mock

    private LeadTimeRepository leadTimeRepo;

    @Mock

    private MinMaxSitePrefRepository mmspRepo;

    @Mock

    private JdbcTemplate jdbcTemplate;

    @InjectMocks

    private LeadTimeServiceImpl leadTimeService;

    private static final Logger LOGGER = LoggerFactory.getLogger(LeadTimeServiceImplUnitTest.class);

    private final String leadTimeTestUrl = "http://example.com/testUrl";

    @BeforeEach

    public void setup(){

        MockitoAnnotations.initMocks(this);

        ReflectionTestUtils.setField(leadTimeService,"leadTimeTestUrl" , leadTimeTestUrl);

        ReflectionTestUtils.setField(leadTimeService,"restTemplate" , restTemplate);

        ReflectionTestUtils.setField(leadTimeService,"jdbcTemplate" , jdbcTemplate);

    }

    @Test

    public void testValidateOnPremCloudConnection_Success() throws Exception {

        List<String> sites = Arrays.asList("1", "2","3");

        String token = "dummyToken";

        String jsonResponse = "{ \"content\": { \"key\": \"value\" } }";

        HttpHeaders headers = new HttpHeaders();

        headers.add("Authorization", "Bearer ".concat(token));

        HttpEntity<Object> entity = new HttpEntity<>(headers);

        Map<String, List<FiledLineGroupCode>> leadTimeDataMap = new HashMap<>();

//        leadTimeDataMap.put("1", Collections.singletonList(new FiledLineGroupCode("FLA1", 5, 1)));

        LeadTimeData leadTimeData = LeadTimeData.builder().leadTimeData(leadTimeDataMap).build();


        when(gcpAuthenticationService.getToken()).thenReturn("dummyToken");

        when(restTemplate.exchange(anyString(), any(), any(), eq(String.class)))

                .thenReturn(new ResponseEntity<>("{\"content\": {\"1\": [{\"fldLineAbbr\": \"FLA1\", \"vndrLeadTime\": 5}]}}", HttpStatus.OK));

        when(siteRepo.findSiteId(any())).thenReturn(100);

        Integer p = 1;

        when(leadTimeRepo.getMinMaxClacLnByFieldLine(anyString())).thenReturn(getminmaxclacln());


        LeadTimeData result = leadTimeService.validateOnPremCloudConnection(sites);

        assertNotNull(result);

        assertEquals(1, result.getLeadTimeData().size());

//        assertTrue(result.getLeadTimeData().containsKey("1"));

//        assertTrue(result.getLeadTimeData().containsKey("2"));

//        assertTrue(result.getLeadTimeData().containsKey("3"));

        verify(jdbcTemplate, times(1)).update(anyString());

        verify(mmspRepo, times(1)).save(any());

    }

    public MinMaxClacLn getminmaxclacln(){

        MinMaxClacLn minMaxClacLn = new MinMaxClacLn();

        minMaxClacLn.setLineId(1);

        minMaxClacLn.setCalcFormulaId(1);

        minMaxClacLn.setFieldLine("FLA1");

        minMaxClacLn.setActiveFlg(1);

        return minMaxClacLn;

    }

    @Test

    public void testValidateOnPremCloudConnection_Exception() throws Exception {

        List<String> sites = Arrays.asList("site1", "site2");

        String token = "dummyToken";

        when(gcpAuthenticationService.getToken()).thenReturn(token);

        when(restTemplate.exchange(anyString(), eq(HttpMethod.GET), any(HttpEntity.class), eq(String.class)))

                .thenThrow(new RuntimeException("Exception in RestTemplate"));

        LeadTimeData result = leadTimeService.validateOnPremCloudConnection(sites);

        assertNull(result);

        verify(gcpAuthenticationService, times(1)).getToken();

        verify(restTemplate, times(1)).exchange(anyString(), eq(HttpMethod.GET), any(HttpEntity.class), eq(String.class));

    }


    @Test

    public void testLeadTimeServiceImplConstructor() {

        assertDoesNotThrow(() -> {

            new LeadTimeServiceImpl();

        }, "Constructor threw an unexpected exception");

    }



}