package com.napa.pulse.service.impl;

import com.napa.pulse.entity.pulseui.TrainingResource;
import com.napa.pulse.repository.TrainingResourceRepository;
import org.junit.Test;
import org.junit.jupiter.api.BeforeEach;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.mockito.junit.MockitoJUnitRunner;
import org.springframework.boot.test.context.SpringBootTest;

import java.util.Collections;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.when;

/**
 * @author Aqueel Ahmad
 * 29/08/2022
 */
@SpringBootTest
@RunWith ( MockitoJUnitRunner.class )
public class TrainingResourceServiceImplUnitTest {

    @InjectMocks
    TrainingResourceServiceImpl trainingResourceImpl;

    @Mock
    TrainingResource trainingResource;

    @Mock
    TrainingResourceRepository trainingResourceRepository;

    @BeforeEach
    public void setup ( ) {
        MockitoAnnotations.initMocks(this);
        trainingResourceRepository = Mockito.mock(TrainingResourceRepository.class);
        trainingResourceImpl = new TrainingResourceServiceImpl(trainingResourceRepository);

    }

    @Test
    public void test_CreateNewItem ( ) {
        trainingResourceRepository = Mockito.mock(TrainingResourceRepository.class);
        trainingResourceImpl = new TrainingResourceServiceImpl(trainingResourceRepository);
        TrainingResource expected = new TrainingResource();
        expected.setIsActive(1);
        when(trainingResourceRepository.save(any())).thenReturn(expected);
        trainingResource = trainingResourceImpl.createNewItem(expected);
        assertThat(trainingResource).isNotNull();

    }

    @Test
    public void test_getAllResources ( ) {
        trainingResourceRepository = Mockito.mock(TrainingResourceRepository.class);
        trainingResourceImpl = new TrainingResourceServiceImpl(trainingResourceRepository);
        TrainingResource trainingResource = new TrainingResource();
        when(trainingResourceRepository.findByIsActiveOrderByDateDesc(anyInt(), anyString())).thenReturn(Collections.singleton(trainingResource));
        Iterable<TrainingResource> trainingResources = trainingResourceImpl.getAllResources("STATIC");
        assertThat(trainingResources).isNotNull();
    }

    @Test
    public void test_getResourceLink ( ) {
        String expected = "xyz";
        trainingResourceRepository = Mockito.mock(TrainingResourceRepository.class);
        trainingResourceImpl = new TrainingResourceServiceImpl(trainingResourceRepository);
        when(trainingResourceRepository.getResourceUrl(anyString(), anyString())).thenReturn(expected);
        String link = trainingResourceImpl.getResourceLink("fleets", "STATIC");
        assertThat(link).isNotNull();

    }

    @Test
    public void test_updateResource () {
        TrainingResource expected = new TrainingResource();
        trainingResourceRepository = Mockito.mock(TrainingResourceRepository.class);
        trainingResourceImpl = new TrainingResourceServiceImpl(trainingResourceRepository);
        when(trainingResourceRepository.findById(any())).thenReturn(java.util.Optional.of(expected));
        trainingResource.setResourceGroup(anyString());
        trainingResource.setTitle(anyString());
        trainingResource.setResourceGroup(anyString());
        when(trainingResourceRepository.save(any())).thenReturn(expected);
        trainingResource = trainingResourceImpl.updateResource(expected);
        assertThat(trainingResource).isNotNull();

    }
    @Test
    public void test_deleteResources()
    {  TrainingResource expected = new TrainingResource();
        trainingResourceRepository = Mockito.mock(TrainingResourceRepository.class);
        trainingResourceImpl = new TrainingResourceServiceImpl(trainingResourceRepository);
        when(trainingResourceRepository.findById(any())).thenReturn(java.util.Optional.of(expected));
        trainingResource.setIsActive(anyInt());
        when(trainingResourceRepository.save(any())).thenReturn(expected);
        trainingResource = trainingResourceImpl.deleteResources(anyInt());
        assertThat(trainingResource).isNotNull();

    }

}
